/*  File Cleaner Header File.
	An application to clean files in a specified folder or in the Temporary
    Internet Files using deleting, wiping, and DES wiping.
    (c) 2004, Alfred P. Reaud
    Created: Thursday, November 11, 2004
    Current: Tuesday, December 7, 2004
    Version: 1.0.1.3

	This program is free software; you can redistribute it and/or modify
  	it under the terms of the GNU General Public License as published by
  	the Free Software Foundation; either version 2 of the License, or
  	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
  	but WITHOUT ANY WARRANTY; without even the implied warranty of
  	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
  	along with this program; if not, 
    browse to http://www.gnu.org/copyleft/gpl.html
    or write to the:
         Free Software Foundation, Inc.,
         59 Temple Place - Suite 330,
         Boston, MA 02111-1307, USA 

   This software was developed using Dev-C++ 4.9.9.0, downloadable at
   http://www.bloodshed.net/dev/index.html
   
   Required packages:
  	    FLTK.DevPak (Fast Light Tool Kit)
  	    
   Required Libraries (in the Project Options -> Parameters -> Linker), with
   {default} being whatever location you have the compiler installed in:
       {default}/Dev-Cpp/lib/libfltk.a
       {default}/Dev-Cpp/lib/libole32.a
       {default}/Dev-Cpp/lib/libuuid.a
       {default}/Dev-Cpp/lib/libwsock32.a
*/

using namespace std;

// Standard C includes.
#include <sys/stat.h>
#include <direct.h>

// C++ includes.
#include <cstdio>
#include <cmath>
#include <fstream>

#ifndef FileCleaner_h
#define FileCleaner_h

// FLTK includes.
#include <FL/Fl.H>
#include <FL/Fl_Ask.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Check_Button.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Light_Button.H>
#include <FL/Fl_Progress.H>
#include <FL/Fl_Round_Button.H>
#include <FL/Fl_Text_Buffer.H>
#include <FL/Fl_Text_Display.H>
#endif

// Folder and file definitions.
#define BASE_TEN 10
#define MAX_FOLDER_TEXT_SIZE 248
#define NULL_CHAR 0x00
#define NUMBER_BUFFER_SIZE 32
#define PROHIB_COUNT 7
#define RECURSE_MAX_COUNT 5
#define SEARCH_COUNT 5
#define TEXT_BUFFER_SIZE 256

// Estimated time constants.
#define TIMEPERDELETE 3.586E-003
#define TIMEPERWIPE 36.658E-003
#define TIMEPERDESWIPE 139.01E-003
#define SECONDSINMIN 60.0
#define ESTTIMELIM 100E-003
#define SHOWMIN 90.0


typedef struct {	// Semaphore structure of boolean flags.
    bool abortflag;			// User requests processing abort.
    bool doTIFflag;			// Process temporary internet file folders.
    bool enabledeleteflag;	// Enable file deleting.
    bool enablewipeflag;	// Enable normal file wiping.
    bool enableDESflag;		// Enable DES file wiping.
    bool inloopflag;		// Application is in a loop, used for process bar updating.
    bool statechangedflag;	// Window state changed, used for widget redrawing.
    bool stoprecurseflag;	// Stop folder search recursion.
} Flag;

typedef struct {	// Semaphore structure of parameters.
    char *useFolder;		// Folder to use if not operating on temporary internet files.
    long accesserrorcount;	// The count of access errors.
    long deniedcount;		// The count of read only errors.
    long fileinusecount;	// The count of files that where found in use by another application.
    long othererrcount;		// The count of any other errors.
    int recursecount;		// The count of folder search recursion calls.
    int samenamecount;		// The count of times we found the same name.
} Value;

typedef struct {	// Semaphore structure of progress bar parameters.
	float count;	// The current operation count.
	float divider;	// The total operation count.
} Prog;

// Global string constants.  Assume Windows XP.
static const string TIFsearch[SEARCH_COUNT] = { "Temporary Internet Files", "History", "Cookies", 
									"Content.IE5", "History.IE5"};
/*  The following are prohibited folders to operate on.  This is for accidents,
    uneducated, and/or malicious users.
*/
static const string Prohibited[PROHIB_COUNT] = { "c:\\", "drivers", "windows", 
									"winnt", "system", "documents and settings", 
         							"program files" };
static const string prohibprepath = "The default folder ";
static const string prohibpostpath = "\nis actually ";
static const string prohibtext = "\nis a prohibited folder, operations are not allowed here!";
static const string prohibtextdef = "!\nThis is a prohibited folder, operations are not allowed here!";
// The default start for temporary internet file searching.
static const string TIFsearchStart = "c:\\Documents and Settings";
static const string quote = "\"";

// Graphical User Interface CLASS ----------------------------------------------
class GUI {
    private:
        char useFolderbuffer[TEXT_BUFFER_SIZE];	// Character buffer for useFolder text.
        string prompt;				// User dialog buffer.
        Prog progress;				// Application progress structure.
        Fl_Text_Buffer textbuffer;	// FLTK text widget buffer.

    	// FLTK widget pointers.
        Fl_Button *abortCommandButton;
        Fl_Button *cancelCommandButton;
        Fl_Light_Button *startCommandButton;
        Fl_Group *commandbuttonGroup;
        Fl_Text_Display *statusTextDisplay;

    	// Privete procedures.
    	void make_window ( void );			// Create widget window.

   	public:
   	    string appIdleprompt;	// Application idle dialog text.
   	    string init_appIdleprompt;

        // Public widget pointers.
        Fl_Progress *applicationProgressBar;
        Fl_Round_Button *doTIFRadioButton;
        Fl_Double_Window *mainWindow;
        Fl_Check_Button *enabledeleteCheckBox;
        Fl_Check_Button *enablewipeCheckBox;
        Fl_Check_Button *enableDESCheckBox;
        Fl_Input *usefolderTextBox;

   	    GUI (void);				// Class constructor.

   	    void clear_progress ( void );		// Clear progress bar.
   	    void disableWidgets(void);			// Disable window widgets.
   	    void enableWidgets(void);			// Enable window widgets.
   	    void set_prog_divider ( float );	// Set progress bar 100% value.
   	    void update_progress( void );		// Update progress bar by 1 unit.
   	    void write_textDisplay ( string );	// Write status information. 
};


// Code CLASS ------------------------------------------------------------------
class CODE {   
    private:
        // Number to character conversion buffers.
        char numberbufferfloat[NUMBER_BUFFER_SIZE];
        char numberbufferint[NUMBER_BUFFER_SIZE];
        char numberbufferlong[NUMBER_BUFFER_SIZE];
        // Number to character conversion pointers.
        char *numberptrf;
        char *numberptri;
        char *numberptrl;

        string prompt;				// User dialog buffer.
        string foldersfound[SEARCH_COUNT];
        string filesfound[SEARCH_COUNT];

        // Private procedures.
        void process_common( long, string);
        void process_named(void);
        void process_TIF(void);

	public:
	    CODE (void);	// Class constructor.
	    
		void process_ok (void);		// Process ok button.
};


// Utility CLASS ---------------------------------------------------------------
class UTIL { // This class incorporates the utilities section of the application.
    private:
        string prompt;				// User dialog buffer.

        bool bad_flush (int, string);
        int force_flush (FILE *);
        int found_file_process (char *, string , string *);
        void no_file_alert (string);
        long recurse_search_for_folders (string, string, string, string *);
        bool write_data (bool, char, long, FILE *);

    public:
        UTIL ( void );

        long delete_files (long, string);
        long recurse_search_for_folders (string, string, string *);
        long search_for_files (string, string, string *);
        long wipe_files (bool, long, string);
};

// Global structures.
extern Flag semaphore;
extern Value parameters;

// Class declerations for each module.
extern GUI gui;		// Declared in FileCleaner_Main.cpp.
extern CODE code;	// Declared in FileCleaner_Code.cpp.
extern UTIL util;	// Declared in FileCleaner_Util.cpp.
